/*****************************************************************************
**
**  mill_pocket.c
**
**  Description:
**     Contains Unigraphics entry points for the application.
**
*****************************************************************************/

/*  Include files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <fcntl.h>
#include <errno.h>
#include <math.h>
#include <ctype.h>
#include <wchar.h>
#include <uf.h>
#include <uf_cfi.h>
#include <uf_ui.h>
#include <uf_part.h>
#include <uf_object_types.h>
#include <uf_disp.h>
#include <uf_modl.h>
#include <uf_curve.h>
#include <uf_defs.h>
#include <uf_exit.h>
#include <uf_csys.h>
#include <uf_styler.h>
#include <uf_mb.h>
#include <uf_cam.h>
#include <uf_udop.h>
#include <uf_oper.h>
#include <uf_assem.h> 
#include <uf_obj.h>
#include <uf_modl_blends.h>
#include <uf_modl_form_features.h>
#include <uf_modl_datum_features.h>
#include <uf_modl_freeform.h>
#include <uf_modl_grooves.h>
#include <uf_modl_holes.h>
#include <uf_modl_hollow.h>
#include <uf_modl_pads.h>
#include <uf_modl_pockets.h>
#include <uf_modl_primitives.h>
#include <uf_modl_simplify.h>
#include <uf_modl_slots.h>
#include <uf_modl_sweep.h>
#include <uf_modl_taper.h>
#include <uf_modl_types.h>
#include <uf_bound.h>

void get_error(int irc);
void unhighlight_objects(int count, tag_p_t object_p_t);
void print_to_listing_windows(char str[256]);
int select_object(tag_t part_tag);

extern DllExport void ufusr(char *parm, int *returnCode, int rlen)
{
   int irc;
   tag_t part_tag;

   irc = UF_initialize();
	if(irc) get_error(irc);
   part_tag = UF_ASSEM_ask_work_part();
   select_object(part_tag);
   irc = UF_terminate();
   if(irc) get_error(irc);
}

#include "utility.h"

int select_object(tag_t part_tag)
{
   char *feature_type;
   int enttype = UF_feature_type;
   tag_t feature = 0;
   int number_object = 0;
   int edit = 0;
   char *diameter = "NULL", *depth = "NULL";
   char *floor_rad = "NULL", *taper_angle = "NULL";
   char t_str[256] = "\n";
   char *str = "NULL";
   uf_list_p_t feature_list, feature_list_1, temp_list;
   int number_edges, number_edges_1;
   int i, j, k, l, irc;
   int number_member = 0;
   tag_t edge_tag, edge_tag_1;
   tag_t target_face_eid, tool_face_eid, dir_ref;
   logical ref_is_horizontal;
   tag_t bound_tag, view_tag;
   UF_BOUND_tolerance_t  tolerance;
   UF_BOUND_object_t object_list[8];
   tag_t tmp_tag[8];
   int open_closed_flag;
   int guess1_given = 0, guess2_given = 0;
   double guess1[3], guess2[3], pt_on_ent1[3], pt_on_ent2[3];
   double min_dist, accuracy;
   double d_diameter, d_depth, d_floor_rad, d_taper_angle;
   do
   {
      UF_OBJ_cycle_objs_in_part(part_tag, enttype, &feature);
      if(feature == NULL_TAG) break;
      /* Get the feature type */
      UF_MODL_ask_feat_type(feature, &feature_type);
//      if((strstr(feature_type, "CYL_POCKET") != NULL)) 
      if((strstr(feature_type, "RECT_POCKET") != NULL))
      {
         print_to_listing_windows(feature_type);
         number_member = 0;
         number_object++;
//         UF_MODL_ask_cyl_pocket_parms(feature, edit, 
//               &diameter, &depth, &floor_rad, &taper_angle);
//         print_to_listing_windows(diameter);
//         print_to_listing_windows(depth);
//         print_to_listing_windows(floor_rad);
//         print_to_listing_windows(taper_angle);
//         str = strstr(diameter,"=");
//         sscanf(str+1,"%lf", &d_diameter);
//         str = strstr(depth,"=");
//         sscanf(str+1,"%lf", &d_depth);
//         str = strstr(floor_rad,"=");
//         sscanf(str+1,"%lf", &d_floor_rad);
//         str = strstr(taper_angle,"=");
//         sscanf(str+1,"%lf", &d_taper_angle);
//         sprintf(t_str, "D=%.3lf Depth=%.3lf Floor_rad=%.3lf T_a=%.3lf",         
//            d_diameter, d_depth, d_floor_rad, d_taper_angle);
//         print_to_listing_windows(t_str);
         irc = UF_MODL_ask_feat_edges(feature, &feature_list);
         if(irc) get_error(irc);
         irc = UF_MODL_ask_list_count(feature_list, &number_edges);
         if(irc) get_error(irc);
         irc = UF_MODL_ask_link_faces(feature, &target_face_eid, &tool_face_eid, 
               &dir_ref, &ref_is_horizontal);
         if(irc) get_error(irc);
//         UF_DISP_set_highlight(target_face_eid, 1);
         irc = UF_MODL_ask_face_edges(target_face_eid, &feature_list_1);
         if(irc) get_error(irc);
         irc = UF_MODL_ask_list_count(feature_list_1, &number_edges_1);
         if(irc) get_error(irc);
//         number_member = 0;
         for(j = 0; j < number_edges; j++)
         {  
            irc = UF_MODL_ask_list_item(feature_list, j, &edge_tag);
            if(irc) get_error(irc);
            for(k = 0; k < number_edges_1; k++)
            {
               irc = UF_MODL_ask_list_item(feature_list_1, k, &edge_tag_1);
               if(irc) get_error(irc);
               if(edge_tag == edge_tag_1 && j == 0) 
               {
                  object_list[number_member].on_tangent_to_flag = UF_BOUND_TANGENT_TO;
                  object_list[number_member].object_tag = edge_tag_1;
                  number_member++;
               }
               else if(edge_tag == edge_tag_1 && j != 0)
               {
                  l = k;
                  irc = UF_MODL_ask_list_item(feature_list_1, l+1, tmp_tag);
                  if(irc) get_error(irc);
                  irc = UF_MODL_ask_minimum_dist_2(edge_tag_1, tmp_tag, 
                        guess1_given, guess1, guess2_given, guess2, 
                        &min_dist, pt_on_ent1, pt_on_ent2, &accuracy);
                  if(irc) get_error(irc);
sprintf(t_str, "Dist=%.3lf", min_dist);
print_to_listing_windows(t_str);
                  do
                  {
                     print_to_listing_windows("DO");
                     irc = UF_MODL_ask_minimum_dist_2(edge_tag_1, tmp_tag, 
                           guess1_given, guess1, guess2_given, guess2, 
                           &min_dist, pt_on_ent1, pt_on_ent2, &accuracy);
                     if(irc) get_error(irc);

                     if(min_dist == 0.0)
                     {
                        object_list[number_member].on_tangent_to_flag = UF_BOUND_TANGENT_TO;
                        object_list[number_member].object_tag = edge_tag_1;
                        number_member++;
                     }
                     else 
                     {
                        l++;
                        irc = UF_MODL_ask_list_item(feature_list_1, l, &tmp_tag);
                        if(irc) get_error(irc);
                     }
                  }while(min_dist != 0.0);
               }
            }
//            sprintf(t_str, "Dist=%.3lf", min_dist);
//            print_to_listing_windows(t_str);
         }
   for(i = 0; i < number_member; i++)
   {
      sprintf(t_str, "TAG=%d", object_list[i].object_tag);
      print_to_listing_windows(t_str);
   }

//    print_to_listing_windows("NEXT---------------NEXT");


/*
         irc = UF_MODL_create_list(&temp_list);
         if(irc) get_error(irc);
         for(i = 0; i < number_member; i++)
         {
            
            irc = UF_MODL_put_list_item(temp_list, object_list[i].object_tag);
            if(irc) get_error(irc);
            sprintf(t_str, "Num=%d", object_list[i].object_tag);
            print_to_listing_windows(t_str);
         }
 irc = UF_MODL_ask_minimum_dist_2(object_list[0].object_tag, object_list[1].object_tag, guess1_given, 
      guess1, guess2_given, guess2, &min_dist, pt_on_ent1, pt_on_ent2, &accuracy);
      sprintf(t_str, "Distance = %.3lf", min_dist);
      print_to_listing_windows(t_str);
         for(i = 0; i <= number_member-2; i++)
         {
            if(i == 0) 
            {   
               tmp_tag[i] = object_list[i].object_tag;
            }
            else 
            {
               tmp_tag[i] = object_list[i].object_tag;
               irc = UF_MODL_ask_minimum_dist_2(object_list[i+1].object_tag, 
                  tmp_tag[i], guess1_given, guess1, guess2_given, 
                  guess2, &min_dist, pt_on_ent1, pt_on_ent2, &accuracy);
               if(irc) get_error(irc);
               sprintf(t_str, "Distance_1 = %.3lf", min_dist);
               print_to_listing_windows(t_str);
               if(min_dist == 0.0)
               {
                  tmp_tag[i] = object_list[i].object_tag;
               }
               else
               {
                  j = i;
                  while(min_dist != 0.0)
                  {
                     irc = UF_MODL_ask_minimum_dist(object_list[i++].object_tag, tmp_tag[j], guess1_given, 
                        guess1, guess2_given, guess2, &min_dist, 
                        pt_on_ent1, pt_on_ent2);
                     if(irc) get_error(irc);
sprintf(t_str, "Distance_2 = %.3lf", min_dist);
print_to_listing_windows(t_str);
sprintf(t_str, "TMP_TAG = %d", object_list[i].object_tag);
print_to_listing_windows(t_str);

                  }
                  tmp_tag[j] = object_list[i].object_tag;
                  i = j;
sprintf(t_str, "TMP_TAG = %d", tmp_tag[i]);
print_to_listing_windows(t_str);
               }
            }
         } 
         for(i = 0; i <= number_member; i++)
         {
            object_list[i].object_tag = tmp_tag[i];
         }
         print_to_listing_windows("NEXT---------------NEXT");
         for(i = 0; i < number_member; i++)
         {
            sprintf(t_str, "Num=%d", tmp_tag[i]);
            print_to_listing_windows(t_str);
         }
*/
//         open_closed_flag = UF_BOUND_CLOSED;
//         view_tag = NULL_TAG;
//         tolerance.tol_specified = UF_BOUND_NOT_SPECIFIED;
//         irc = UF_BOUND_create_boundary(open_closed_flag, view_tag,
//            tolerance, number_member, object_list, &bound_tag);
//         if(irc) get_error(irc);
         UF_MODL_delete_list(&feature_list);
         UF_MODL_delete_list(&feature_list_1);
      }
/*      else if((strstr(feature_type, "RECT_POCKET") != NULL))
      {
         print_to_listing_windows(feature_type);
         number_object++;
      }
      else if((strstr(feature_type, "RECT_SLOT") != NULL))
      {
         print_to_listing_windows(feature_type);
         number_object++;
      }
      else if((strstr(feature_type, "BALL_END_SLOT") != NULL))
      {
         print_to_listing_windows(feature_type);
         number_object++;
      }
      else if((strstr(feature_type, "U_SLOT") != NULL))
      {
         print_to_listing_windows(feature_type);
         number_object++;
      }
      else if((strstr(feature_type, "T_SLOT") != NULL))
      {
         print_to_listing_windows(feature_type);
         number_object++;
      }
      else if((strstr(feature_type, "DOVE_TAIL_SLOT") != NULL))
      {
         print_to_listing_windows(feature_type);
         number_object++;
      }
*/
   } while(feature);
   return number_object;   
}
