Date:  18-APR-2005
Subject:  Sample CAM API program to create part boundary

/*****************************************************************************
**
** create_part_boundary.c
**
** Description:
**     Contains Unigraphics entry points for the application.
**
** Need partfile with a CAM setup and a planar mill operation or
** MILL_BND geometry group
**
*****************************************************************************/

/* Include files */
#include <stdio.h>
#include <uf.h>
#include <uf_ui.h>
#include <string.h>
#include <uf_cam.h>
#include <uf_lib.h>
#include <uf_param.h>
#include <uf_param_indices.h>
#include <uf_part.h>
#include <uf_obj.h>
#include <uf_obj_types.h>
#include <uf_ui_ont.h>
#include <uf_ui.h>
#include <uf_so.h>
#include <uf_oper.h>
#include <uf_setup.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include <uf_mtx.h>
#include <uf_curve.h>

#include <uf_cambnd.h>
#include <uf_disp.h>



#define UF_CALL(X) (report_error( __FILE__, __LINE__, #X, (X)))

static int report_error( char *file, int line, char *call, int irc)
{
    if (irc)
    {
        char err[133],
             msg[133];

        sprintf(msg, "*** ERROR code %d at line %d in %s:\n+++ ",
            irc, line, file);
        UF_get_fail_message(irc, err);

        UF_print_syslog(msg, FALSE);
        UF_print_syslog(err, FALSE);
        UF_print_syslog("\n", FALSE);
        UF_print_syslog(call, FALSE);
        UF_print_syslog(";\n", FALSE);

        if (!UF_UI_open_listing_window())
        {
            UF_UI_write_listing_window(msg);
            UF_UI_write_listing_window(err);
            UF_UI_write_listing_window("\n");
            UF_UI_write_listing_window(call);
            UF_UI_write_listing_window(";\n");
        }
    }

    return(irc);
}

#define WRITE(X)   UF_UI_open_listing_window(); UF_UI_write_listing_window(X)


#define     MAX_BND      20

void write_string_to_listing_window(char *title, char *string)
{
    char
        msg[MAX_LINE_SIZE+1];

    UF_CALL(UF_UI_open_listing_window());
    sprintf(msg, "%s = %s\n", title, string);
    UF_CALL(UF_UI_write_listing_window(msg));
}

/*******************************************************************/
static UF_CAMBND_boundary_data_p_t create_and_init_bnd_data
(
    UF_CAM_boundary_type_t   type,
    int                      plane_type,
    double                   *origin,
    double                   *matrix,
    UF_CAM_material_side_t   material_side,
    int                      ignore_holes,
    int                      ignore_islands,
    int                      ignore_chamfers,
    UF_CAMBND_app_data_p_t   app_data
);

static UF_CAMBND_app_data_p_t create_and_init_bnd_app_data(void);


static int init_proc1(UF_UI_selection_p_t select, void *user_data);
static int init_proc2(UF_UI_selection_p_t select, void *user_data);

static logical ask_yes_or_no(char *title, char *message)
{
    int
        resp;
    UF_UI_message_buttons_t
        buttons = { TRUE, FALSE, TRUE, "Yes", NULL, "No", 1, 0, 2 };

    UF_CALL(UF_UI_message_dialog(title, UF_UI_MESSAGE_QUESTION, &message, 1,
        TRUE, &buttons, &resp));

    return (2 - resp);
}


static UF_CAMBND_boundary_data_p_t create_and_init_bnd_data
(
    UF_CAM_boundary_type_t   type,
    int                      plane_type,
    double                   *origin,
    double                   *matrix,
    UF_CAM_material_side_t   material_side,
    int                      ignore_holes,
    int                      ignore_islands,
    int                      ignore_chamfers,
    UF_CAMBND_app_data_p_t   app_data
)
{
    UF_CAMBND_boundary_data_p_t bnd_data;
    int                      i, err_code;
	char					 msg[133];

    bnd_data = (UF_CAMBND_boundary_data_p_t) UF_allocate_memory(
                              sizeof(UF_CAMBND_boundary_data_t), &err_code);

	sprintf(msg," creating and init bnd_data\n");
	WRITE(msg);

    bnd_data->boundary_type = type;
    bnd_data->plane_type = plane_type ;
    for (i=0; i<3; i++)
        bnd_data->origin[i] = origin[i] ;
    for (i=0; i<9; i++)
        bnd_data->matrix[i] = matrix[i] ;
    bnd_data->material_side = material_side;
    bnd_data->ignore_holes = ignore_holes ;
    bnd_data->ignore_islands = ignore_islands ; 
    bnd_data->ignore_chamfers = ignore_chamfers ;    
    bnd_data->app_data = app_data;

    return(bnd_data);
}

static UF_CAMBND_app_data_p_t create_and_init_bnd_app_data(void)
{
    UF_CAMBND_app_data_p_t  app_data; 
    int                     err_code;
   
	char msg[133];

    app_data = (UF_CAMBND_app_data_p_t) UF_allocate_memory(
                                 sizeof(UF_CAMBND_app_data_t), &err_code);

    if (err_code)
		return(NULL);

	sprintf(msg," creating and init bnd_app_data\n");
	WRITE(msg);

    /* Set flags. */
    app_data->has_stock = 1;
    app_data->has_tolerances = 0 ;
    app_data->has_feedrate = 0 ;
    app_data->has_blank_distance = 0;
// if creating this for a GEOM parent 
// app_data->has_tool_position MUST be set to 0
    app_data->has_tool_position = 0;
    
    /* Set values. */
    app_data->stock = 0.1 ;
    app_data->tolerances[0] = 0.1 ;
    app_data->tolerances[1] = 0.1 ;
    app_data->feedrate_unit = UF_CAM_feedrate_unit_per_minute ;
    app_data->feedrate_value = 0.1 ;
    app_data->blank_distance = 0.0 ;
    app_data->tool_position = UF_CAM_tool_position_tanto;

    return(app_data);
}

/* Selection initialization procedures.*/
static int init_proc1
(
    UF_UI_selection_p_t select,
    void* user_data
)
{
    int num_triples = 1;
    UF_UI_mask_t mask_triples[] =
	{UF_solid_type, 0, UF_UI_SEL_FEATURE_ANY_FACE};

    /* enable only lines and edges */
    if((UF_UI_set_sel_mask(select,
                           UF_UI_SEL_MASK_CLEAR_AND_ENABLE_SPECIFIC,
                           num_triples, mask_triples)) == 0)
    {
        return (UF_UI_SEL_SUCCESS);
    }
    else
    {
        return (UF_UI_SEL_FAILURE);
    }
}

static int init_proc2
(
    UF_UI_selection_p_t select,
    void* user_data
)
{
    int num_triples = 1;
    UF_UI_mask_t mask_triples[] = 
	                      {UF_solid_type, 0, UF_UI_SEL_FEATURE_ANY_EDGE};

    /* enable only lines and edges */
    if((UF_UI_set_sel_mask(select,
                           UF_UI_SEL_MASK_CLEAR_AND_ENABLE_SPECIFIC,
                           num_triples, mask_triples)) == 0)
    {
        return (UF_UI_SEL_SUCCESS);
    }
    else
    {
        return (UF_UI_SEL_FAILURE);
    }
}


static void do_it(void)
{    
   int          i,
				j,
				object_count,
				type,
				subtype,
				entity_count,
				response;

   double		cursor[3],
				origin[3] = {0,0,2} , matrix[9]= {1,0,0,0,1,0,0,0,1};

   char			msg[133],
				cue_face[]   = "Select faces",
				cue_edge[]   = "Select edges for open BND",
				title[]      = "Boundary Selection";

   tag_t        part_tag = UF_PART_ask_display_part( ),
				*objects,
				entity_list[MAX_BND],
				eid,
				view;


    UF_CAMBND_boundary_data_p_t  bnd_data,
								 bnd_data_list[MAX_BND];

    UF_CAMBND_app_data_p_t		app_data_list[MAX_BND]; 


    if( part_tag != null_tag )
    {
     /* Get the highlighted/selected objects from Navigation Tool. */
     UF_UI_ONT_ask_selected_nodes( &object_count, &objects ); 

     if (object_count > 0)
     {
	  entity_count = 0;
	  while (TRUE)
      {
	// ************ This is for seclecting faces 
        UF_UI_select_with_single_dialog(cue_face, title,
                                 UF_UI_SEL_SCOPE_NO_CHANGE, init_proc1, NULL,
                                  &response, &eid, cursor, &view);

         if (response == UF_UI_OBJECT_SELECTED)
         {                    
            entity_list[entity_count] = eid;

/* Allocate the memory for the boundary data of an item without app data.  */
/* NOTE: there is no manual plane selection for faces
/* look at the second argument for create_and_init_bnd_data 1 = automatic  */
/* origin and matrix not used                                              */
            bnd_data_list[entity_count] = 
				          create_and_init_bnd_data(UF_CAM_boundary_type_closed,
                                                1, origin, matrix, 
                                                UF_CAM_material_side_out_right,
                                                 0, 0, 0, NULL);
             entity_count++;
          }
		  else if (response == UF_UI_OK || response == UF_UI_CANCEL || 
                   response == UF_UI_BACK)
          {
                 /* Geometry selection is finished. */
              break;
          }
        }

	/* Append the face boundaries. */
		if (entity_count > 0)
		{
			for(i=0; i<object_count; i++)
			{
			 UF_OBJ_ask_type_and_subtype(objects[i], &type, &subtype);

		/* The object type must be Operation or Geometry Group. */
			  if( type == UF_machining_operation_type ||
				  type == UF_machining_geometry_grp_type)
			  {
				for (j=0; j<entity_count; j++)
				{
                    UF_CALL(UF_CAMBND_append_bnd_from_face(objects[i],
						                    UF_CAM_part, 
                                            entity_list[j], bnd_data_list[j]));
                           
				}
			  }
			} // end of for(i=0; i<object_count; i++)
		 } // if (entity_count > 0)

           /******* Free the boundary data. *********/
		for (i=0; i<entity_count; i++)
		{
			UF_free(bnd_data_list[i]);
		}

		entity_count = 0;

      /***************** Allow the user to select edges. *****************/
		while (TRUE)
		{
		 UF_UI_select_with_single_dialog(cue_edge, title,
                                  UF_UI_SEL_SCOPE_NO_CHANGE, init_proc2, NULL,
                                              &response, &eid, cursor, &view);

		 if (response == UF_UI_OBJECT_SELECTED)
		 {                    
			entity_list[entity_count] = eid;
			app_data_list[entity_count] = create_and_init_bnd_app_data();
			entity_count++;
		 }
		 else if (response == UF_UI_OK || response == UF_UI_CANCEL || 
			      response == UF_UI_BACK)
		{
           /* Geometry selection is finished. */
			sprintf(msg,"Geometry selection is finished\n");
			WRITE(msg);
			break;
		}
	} // end of while


/* Allocate the memory for the boundary data of an item  */
/* NOTE: we are setting plane selection to auto
/* look at the second argument for create_and_init_bnd_data 1 = automatic  */
/* origin and matrix not used                                              */
    if (entity_count > 0)
    {
      bnd_data = create_and_init_bnd_data(UF_CAM_boundary_type_open,
                                                1, origin, matrix, 
                                                UF_CAM_material_side_out_right,
                                                0, 0, 0, NULL);
		sprintf(msg,"object_count is %d\n",object_count);
		WRITE(msg);
        for(i=0; i<object_count; i++)
        {
          UF_OBJ_ask_type_and_subtype(objects[i], &type, &subtype);

           if( type == UF_machining_operation_type || 
               type == UF_machining_geometry_grp_type)
            {
               UF_CAMBND_append_bnd_from_curve(objects[i], 
				                         UF_CAM_part, entity_count, 
                                         entity_list, bnd_data, app_data_list);
             }
         }
            
         /* Free memory. */
          for (i=0; i<entity_count; i++)
          {
             UF_free(app_data_list[i]);
          }
                
                UF_free (bnd_data);
          } 

	} // if object_count > 0

}  // end of if part_tag != null_tag

}
/*ARGSUSED*/
extern DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
    if (UF_CALL(UF_initialize())) return;
    do_it();
    UF_CALL(UF_terminate());
}

extern int ufusr_ask_unload( void )
{
    return( UF_UNLOAD_IMMEDIATELY );
}

